--[[    use in conjunction with updateWeatherSensors.lua Version 0.20180512
        this file contains most of the functions required for the main code and
        must be saved as/at "domoticz dir"/scripts/dzVents/scripts/modules/wusFunctions.lua
 ]]--
 
local self = { }

function self.getCurrentDir()
    local popenString
    local chr = os.tmpname():sub(1,1)
    print (chr)
    if chr == "/" then     -- Linux
        popenString = "echo $PWD 2>&1"
    else
        popenString = "cd "
    end
    local file = assert(io.popen(popenString))
    local popenReturn = file:read()
    return popenReturn,chr
end

function self.fileExists(filename)
	local file = io.open(filename, "r")
	if file ~= nil then
		io.close(file)
		return true
	else
		return false
	end
end
function self.smartCase(text)
    return (text:gsub("^%l", string.upper))
end

function self.barometerForecastInfo(pressure)                                               -- Return text
  if pressure == nil          then return nil                                               -- Should not happen
        elseif pressure < 966   then return INF.BAROTHUNDERSTORM
        elseif pressure < 993   then return INF.BAROCLOUDYRAIN
        elseif pressure < 1007  then return INF.BAROCLOUDY
        elseif pressure < 1013  then return INF.BAROUNSTABLE
        elseif pressure < 1033  then return INF.BAROSTABLE
    end
    return INF.BAROSUNNY
end

function self.getFileDate (filename)
    if filename ~= nil then
        dzW.log("self.getFileDate: " .. filename,dzW.LOG_DEBUG)
    end
    -- Get mod date
    local popenString = "stat -c %Y " .. filename .. " 2>&1"
    local file = assert(io.popen(popenString))
    local lastModifiedSecondsSinceEpoch = file:read()
    if tonumber(lastModifiedSecondsSinceEpoch) ~= nil then
        local modDate = os.date("%c", lastModifiedSecondsSinceEpoch)
        return modDate
    else
        return nil
    end
end

function self.getDataFromFile (filename)
    if filename ~= nil then
        dzW.log("self.getDataFromFile: " .. filename,dzW.LOG_DEBUG)
    end
    -- get data
    local file = assert(io.open(filename,"r"))
    local data = file:read "*all"
    file:close()
    return data
end

function self.dump2File (data,filename)
    if filename ~= nil then
        dzW.log("self.dump2File: " .. filename,dzW.LOG_DEBUG)
    end
    local file = assert(io.open(filename,"w"))
    file:write(data)
    file:close()
end

function self.getModSecondsAgo (filename)
    local popenString = "stat -c %Y " .. filename .. " 2>&1"
    local file = assert(io.popen(popenString))
    local lastModifiedSecondsSinceEpoch = file:read()
    popenString = "stat -c %Y domoticz.db 2>&1"
    file = assert(io.popen(popenString))
    local currentSecondsSinceEpoch = file:read()
    if tonumber(currentSecondsSinceEpoch) ~= nil and tonumber(lastModifiedSecondsSinceEpoch) ~= nil then
        return tonumber(currentSecondsSinceEpoch) - tonumber(lastModifiedSecondsSinceEpoch)
    else
        return 2^1024
    end
end

function self.writeTestData (data,filename)
    if not(self.fileExists(filename)) or tonumber(self.getModSecondsAgo (filename)) > const.TESTSETAGESECONDS then
            self.dump2File(data,filename)
    else
        dzW.log("self.writeTestData: " .. filename .. " too recent.",dzW.LOG_DEBUG)
    end
end

function self.infoLog(text)
    if var.displayInfoLog then
        dzW.log(text, dzW.LOG_INFO)
    end
end

function self.round2(num, numDecimalPlaces)
    return string.format("%." .. (numDecimalPlaces or 0) .. "f", num)
end

function self.leftPad(text,length,char)
    lpad = function (s, l, c)
        local res = string.rep(c or ' ', l - #s) .. s
        return res
    end
    return (lpad(tostring(text),length or const.DEFAULTLENGTH,char or ' '))
end

function self.rightPad(text,length,char)
    rpad = function (s, l, c)
        local res = s .. string.rep(c or ' ', l - #s)
        return res
    end
    return (rpad(tostring(text),length or const.DEFAULTLENGTH,char or ' '))
end

function self.wrapSpaces(wraptext,where,amountBefore,amountAfter)
    local myspaceBefore = " "
    local myspaceAfter  = " "
    if amountBefore ~= nil and amountBefore > 1 then
       myspaceBefore = self.rightPad(" ",amountBefore)
    end
    if amountAfter ~= nil and amountAfter > 1 then
       myspaceAfter = self.rightPad(" ",amountAfter)
    end
    if where == nil or where == const.BWRAP then
        return (myspaceBefore .. wraptext .. myspaceAfter)
    elseif  where == const.EWRAP  then
        return (wraptext .. myspaceAfter)
    elseif  where == const.SWRAP  then
        return (myspaceBefore .. wraptext)
    end
    return(wraptext)
end

function self.isDevice(device)
    if device ~= nil then
        if device.deviceType == const.GENERAL then
            return device.deviceSubType
        else
            return device.deviceType
        end
    else
        return INF.NONE
    end
end

function self.round(number,decimals)
    local factor = 0
    if decimals == nil or decimals == 0 then
        factor = 1
    elseif decimals < 0 then                   -- not allowed
        return nil
    else
        factor = math.pow(10,decimals)
    end
    return ((number*factor) % 1 >= 0.5 and math.ceil(number*factor) or math.floor(number*factor)) / factor
end

function self.windChill(windSpeed, temperature)
    if temperature ==   nil then return -99     -- Should not happen
    elseif windSpeed == nil then return -99     -- Should not happen
    -- chill temperature cannot be calculated for these conditions
    elseif windSpeed < 1.3 or windSpeed > 49 or temperature < -46 or temperature > 17 then return temperature
    end
    return self.round(13.12 + (0.6215 * temperature) - (13.96 * math.pow(windSpeed, 0.16)) + (0.4867 * temperature * math.pow(windSpeed, 0.16)), 1)
end

function self.checkForNil(value,replacement)
    if value ~= nil then
        return value
    end
    return replacement
end

function self.meters2Bf(meters)                   --  return Beaufort base on given windspeed in m/sec
    meters = tonumber(meters)
    if      meters < 0.2  then return 0
    elseif  meters < 1.5  then return 1
    elseif  meters < 3.3  then return 2
    elseif  meters < 5.4  then return 3
    elseif  meters < 7.9  then return 4
    elseif  meters < 10.7 then return 5
    elseif  meters < 13.8 then return 6
    elseif  meters < 17.1 then return 7
    elseif  meters < 20.7 then return 8
    elseif  meters < 24.4 then return 9
    elseif  meters < 28.4 then return 10
    elseif  meters < 32.6 then return 11
    end
    return 12
end

function self.handleWindDevice(device,weatherStation)
    device.updateWind (
                            self.checkForNil(weatherStation.windrichtingGR,0),                        -- bearing in degrees
                            weatherStation.windrichting:gsub("Z","S"):gsub("O","E"),             -- bearing in direction in N, S, E, NNW (this will influence some of the Icons so not translated to dutch)
                            self.checkForNil(tonumber(weatherStation.windsnelheidMS),0),              -- speed in m/s
                            self.checkForNil(tonumber(weatherStation.windstotenMS),0),                -- gust in m/s
                            self.checkForNil(weatherStation.temperatuurGC,-256),                      -- Temp in Celsius
                            self.windChill   (                                               -- windChill
                                            self.checkForNil(tonumber(weatherStation.windsnelheidMS),0),
                                            self.checkForNil(tonumber(weatherStation.temperatuurGC),0 ) ) )
end

function self.handleWindInfo(weatherStation)
    if tonumber(weatherStation.windsnelheidMS) ~= nil then
        if var.displayLanguage ~= "Nederlands" then
            weatherStation.windrichting = weatherStation.windrichting:gsub("Z","S"):gsub("O","E")
        end
        self.infoLog    (   self.rightPad(self.rightPad(INF.WIND,const.LABELPOSITION) ..
                            self.wrapSpaces(INF.ARROW) .. weatherStation.windrichtingGR ..
                            self.wrapSpaces(INF.DEGREES) .. "("  .. weatherStation.windrichting ..  ")," ..
                            self.wrapSpaces(INF.TEMPERATURE) .. weatherStation.temperatuurGC .. INF.CELSIUS ))
        self.infoLog    (   self.rightPad(   self.rightPad  (   INF.WIND,const.LABELPOSITION) ..
                            self.wrapSpaces(INF.ARROW) .. self.wrapSpaces(INF.WINDSPEED,const.EWRAP) ..
                            weatherStation.windsnelheidBF .. " Bf,"  .. self.wrapSpaces(INF.GUST) ..
                            weatherStation.windstotenMS .. " m/s ("  .. self.meters2Bf(weatherStation.windstotenMS) ..
                            " Bf)")  .. var.deviceDefined .. var.backupStation)
     else
        self.infoLog(self.rightPad  (   self.rightPad(INF.WIND,const.LABELPOSITION) .. self.wrapSpaces(INF.ARROW) ..
        INF.NOMEASUREDVALUE )  .. var.deviceDefined )
    end
end

function self.getAlertColor(alertLevel)
    trueColors = { TXT.GREY,            -- 0
                   TXT.GREEN,           -- 1
                   TXT.YELLOW,          -- 2
                   TXT.ORANGE,          -- 3
                   TXT.RED }            -- 4
    return (trueColors[alertLevel+1])
end

function self.RainForecast2MM (rainIntensity)
    -- intensity is converted to mm/hh with formula   -- https://www.buienradar.nl/overbuienradar/gratis-weerdata
    -- intensity = 10^((value-109)/32)
    return math.pow(10,((rainIntensity - 109)/32))
end

function self.RainForecast (rt, requestTime)
    local i = 1
    local totalRainMM = 0
    repeat
        if rt[i] ~= 0 then
            totalRainMM = totalRainMM + self.RainForecast2MM(rt[i])
        end
        i = i + 1
    until i > #rt or (i > (requestTime / 5))                           -- stop iteration when we reached the end of the table or maxtime
    return ( totalRainMM  / 10 )                                       -- one line is 5 minutes
end

function self.BarometerForecastInfo(pressure)                        -- Return text
    if pressure == nil          then return nil                       -- Should not happen
        elseif pressure < 966   then return INF.BAROTHUNDERSTORM
        elseif pressure < 993   then return INF.BAROCLOUDYRAIN
        elseif pressure < 1007  then return INF.BAROCLOUDY
        elseif pressure < 1013  then return INF.BAROUNSTABLE
        elseif pressure < 1033  then return INF.BAROSTABLE
    end
    return INF.BAROSUNNY
end

function self.BarometerForecast(pressure)                            -- Return number
    if pressure == nil          then return nil                       -- Should not happen
        elseif pressure < 966   then return dzW.BARO_THUNDERSTORM
        elseif pressure < 993   then return dzW.BARO_CLOUDYRAIN
        elseif pressure < 1007  then return dzW.BARO_CLOUDY
        elseif pressure < 1013  then return dzW.BARO_UNSTABLE
        elseif pressure < 1033  then return dzW.BARO_STABLE
    end
    return dzW.BARO_SUNNY
end

function self.calculateLux(solarPower,weatherDescription)
    -- This function still needs a lot of work to come even close to being accurate
    -- if at all possible. Current factorlist is for spring / sunny days
    -- Probably we need different lists for other periods of the year.
    local endDayMinutes        = dzW.time.sunsetInMinutes
    local beginDayMinutes      = dzW.time.sunriseInMinutes
    local deltaMinutes          = endDayMinutes - beginDayMinutes
    local minutesSinceMidnight  = dzW.time.secondsSinceMidnight / 60

    local weatherFactor = 1
    if          weatherDescription == "Zwaar bewolkt" then weatherFactor = 3.6
    elseif      weatherDescription == "Zwaar bewolkt en regen" then weatherFactor = 4.05
    elseif      weatherDescription == "Vrijwel onbewolkt (zonnig/helder)" then weatherFactor = 1.4
    elseif      weatherDescription == "Mix van opklaringen en middelbare of lage bewolking" then weatherFactor = 1.9
    elseif      weatherDescription == "Afwisselend bewolkt met lokaal mist(banken)" then weatherFactor = 2.1
    elseif      weatherDescription == "Afwisselend bewolkt met (mogelijk) wat lichte regen" then weatherFactor = 1.63
    elseif      weatherDescription == "Zonnig" then weatherFactor = 0.98
    end
    dzW.log (self.wrapSpaces(DBG.DESCFACTOR,const.EWRAP) .. weatherDescription .. " / " .. weatherFactor,dzW.LOG_DEBUG)
    --Sunny factorList (empirical based)
    local factorList = {170,80,85,90,91,92,94,95,95,95,95,94,90,85,75,70,65,60,25,20,15}
    local timeFactor
    local solarFactor = 1
    if weatherFactor > 1.5 then
        if          solarPower  < 10 then solarFactor = 0
        elseif      solarPower  < 20 then solarFactor = 2
        elseif      solarPower  < 60 then solarFactor = 1
        elseif      solarPower  < 80 then solarFactor = 0.5
        elseif      solarPower  < 100 then solarFactor = 0.4
        else solarFactor = 0.25
        end
    end
    if minutesSinceMidnight > beginDayMinutes and minutesSinceMidnight < endDayMinutes then
        local minutesSinceSunrise = 1 * (minutesSinceMidnight - beginDayMinutes)
        local minutesSinceSunrisePercentage = ( dzW.utils.round(minutesSinceSunrise / deltaMinutes * 100))
        timeFactor = factorList[dzW.utils.round(minutesSinceSunrisePercentage / 5) + 1]
    else
        timeFactor = 0
    end
    return (dzW.utils.round(solarFactor * solarPower * timeFactor * weatherFactor))
end

function self.getHumidityStatus(temperature,humidity,huminfo)
    if humidity == nil or temperature == nil then return nil end  -- Should not happen
    if huminfo ~= nil then
       HUMDRY           = INF.HUMDRY
       HUMWET           = INF.HUMWET
       HUMCOMFORTABLE   = INF.HUMCOMFORTABLE
       HUMNORMAL        = INF.HUMNORMAL
    else
       HUMDRY           = dzW.HUM_DRY
       HUMWET           = dzW.HUM_WET
       HUMCOMFORTABLE   = dzW.HUM_COMFORTABLE
       HUMNORMAL        = dzW.HUM_NORMAL
    end
    if humidity <= 30 then                   return HUMDRY
    elseif humidity >= 70 then                   return HUMWET
    elseif     humidity >= 35       and
            humidity <= 65          and
            temperature >= 22       and
            temperature <= 26              then  return HUMCOMFORTABLE
    else                                         return HUMNORMAL
    end
end

function self.handleGenericDevice(device,type)
    if self.isDevice(device) == type then
        var.devicesDefined = var.devicesDefined + 1
        if device.deviceType == const.GENERAL then
            deviceType = device.deviceSubType
        else
            deviceType = device.deviceType
        end
        var.deviceDefined = self.rightPad(self.wrapSpaces(INF.ARROW) ..
                            self.wrapSpaces(INF.DEVICE,const.EWRAP) .. device.idx .. " (" .. device.name ..
                            ")",const.TYPEPOSITION) .. " ==>> (sub)Type: " .. deviceType
        return true
    else
        var.deviceDefined = self.wrapSpaces(INF.ARROW) .. INF.NODEVICE
        return false
    end
end

function self.handleTextDevice(device,type,text,display,exception)
    if self.handleGenericDevice(device,type) then
        dzW.log(device.name .. self.wrapSpaces(DBG.ISTYPE) .. tostring(self.isDevice(device)), dzW.LOG_DEBUG)
        text = string.sub(text,1,math.min(string.len(text),240))
        if text ~= device.text or device.lastUpdate.minutesAgo > var.forcedUpdateMinutes then
            device.updateText(text)
        end
    end
    if exception ~= nil then
        self.infoLog(self.rightPad(self.rightPad(display,const.DEVICEPOSITION)) .. var.deviceDefined)
     else
        if display ~= INF.NONE then
            self.infoLog(self.rightPad(self.rightPad(display .. text,const.DEVICEPOSITION)) .. var.deviceDefined)
        end
    end
end

function self.distance (lat1,lon1, lat2,lon2, accuracy)
    -- returns distance between two locations
    -- option one (accuracy ~= "exact") using Pythagoras
    -- option two (accuracy == "exact") using radians and stuff
    latFactor = 111.8953
    lonFactor = 68.17
    if accuracy ~= nil then
        return math.acos(math.cos(math.rad(90-lat2))*math.cos(math.rad(90-lat1))+math.sin(math.rad(90-lat2))*math.sin(math.rad(90-lat1))*math.cos(math.rad(lon2-lon1)))*6371
    elseif lat1 ~= nil and lon1 ~= nil and lat2 ~= nil and lon2 ~= nil then
        return math.sqrt(((lat1-lat2)*latFactor)^2 + ((lon1 - lon2)*lonFactor)^2)
    else
        return "0"
    end
end

function self.findHomeWeatherStation ( weatherStations,name )
     local i,indexNumber,km
     local stationIndexNumber
     local minumum = 2^1024
     local lat,lon
     -- local nextWeatherStation = {}

     -- Get the indexNumber using the weatherStation name
     if name ~= nil then
         for indexNumber, nextWeatherStation in ipairs(weatherStations) do
             if weatherStations[indexNumber].stationnaam["#text"] == name then
                return indexNumber
             end
         end
     else  -- Get the lat,lon already in the persistent data
        for indexNumber, nextWeatherStation in ipairs(weatherStations) do
            km = self.distance(  dzW.data.domoticzLatitude,
                            dzW.data.domoticzLongitude,
                            nextWeatherStation.lat,
                            nextWeatherStation.lon, "Y")
            if km < minumum  then
                minumum = km
                stationIndexNumber = indexNumber
            end
        end
        return stationIndexNumber, weatherStations[stationIndexNumber].stationnaam["#text"]
    end
end

function self.findNearbyWeatherStations ( currentWeatherStation )
    local indexNumber
    local lon,lat
    currentWeatherStation.nearbyWeatherStation = {}
    currentWeatherStation.nearbyDistance = {}
    currentWeatherStation.nearbyIndex = {}
    dzW.log("Function findNearbyWeatherStations, stationName (passed to function): " .. currentWeatherStation.stationnaam["#text"] ,dzW.LOG_DEBUG)
    -- Create an additional array within currentWeatherStationRecord containing name and distance to currentWeatherStation
    if  var.useDomoticzLocation then
        lat = dzW.data.domoticzLatitude
        lon = dzW.data.domoticzLongitude
    else
        lon = currentWeatherStation.lon
        lat = currentWeatherStation.lat
    end
    for indexNumber,nextWeatherStation in ipairs(rt) do
        currentWeatherStation.nearbyDistance[indexNumber] = self.distance(      lat,
                                                                                lon,
                                                                                nextWeatherStation.lat,
                                                                                nextWeatherStation.lon, "Y")
        currentWeatherStation.nearbyWeatherStation[indexNumber] = nextWeatherStation.stationnaam["#text"]
        currentWeatherStation.nearbyIndex[indexNumber] = indexNumber
    end
end

function self.sortNearbyWeatherStations ( currentWeatherStation )
    local outerIndexNumber,innerIndexNumber
    local minumum
    currentWeatherStation.sortedNearbyWeatherStation = {}
    currentWeatherStation.sortedNearbyDistance = {}
    currentWeatherStation.sortedNearbyIndex = {}
    for outerIndexNumber,nextWeatherStation in ipairs(rt) do
        minumum = 2^1024
        for innerIndexNumber,nextWeatherStation in ipairs(rt) do
            if currentWeatherStation.nearbyDistance[innerIndexNumber] > 0 and currentWeatherStation.nearbyDistance[innerIndexNumber] < minumum then
                foundMinumumIndex = innerIndexNumber
                minumum = currentWeatherStation.nearbyDistance[innerIndexNumber]
            end
        end
            currentWeatherStation.sortedNearbyWeatherStation[outerIndexNumber] = currentWeatherStation.nearbyWeatherStation[foundMinumumIndex]
            currentWeatherStation.sortedNearbyDistance[outerIndexNumber] = currentWeatherStation.nearbyDistance[foundMinumumIndex]
            currentWeatherStation.sortedNearbyIndex[outerIndexNumber] = currentWeatherStation.nearbyIndex[foundMinumumIndex]
            --Setting to zero to make sure it is not taken into account next iteration
            currentWeatherStation.nearbyDistance[foundMinumumIndex] = 0
    end
    -- Last record is not valid because currentWeatherStation itself is not evaluated
    currentWeatherStation.sortedNearbyDistance[#rt] = 0
    currentWeatherStation.sortedNearbyWeatherStation[#rt] = currentWeatherStation.stationnaam["#text"]
    currentWeatherStation.sortedNearbyIndex[#rt] = 9999
    -- Drop these tables because they are no longer needed.
    currentWeatherStation.nearbyIndex = nil
    currentWeatherStation.nearbyWeatherStation = nil
    currentWeatherStation.nearbyDistance = nil
end

function self.traverseNearbyLocations ( WeatherStation )
    local r, homeLocation
    for r,nextWeatherStation in ipairs(rt) do
        if WeatherStation.sortedNearbyDistance[r]  < var.maxDistanceFromHome and WeatherStation.sortedNearbyDistance[r] ~= 0 then
            if var.useDomoticzLocation then
                homeLocation = "domoticz location"
            else
                homeLocation = WeatherStation.stationnaam["#text"]
            end
            dzW.log(self.rightPad("rank[" .. self.leftPad(tostring(r),2,"0") .. "], ",9) ..
                   self.rightPad("Index[" .. self.leftPad(WeatherStation.sortedNearbyIndex[r],2,"0") .. "] " ,10)..
                   self.rightPad( WeatherStation.sortedNearbyWeatherStation[r],35)   ..
                             " Distance to " .. homeLocation  ..   " ==>> " ..
                            self.round2(WeatherStation.sortedNearbyDistance[r],1) .. " Km.",   dzW.LOG_DEBUG)
        else
            if WeatherStation.sortedNearbyDistance[r] ~= 0 then return ( r - 1 ) end
        end
    end
    return (#WeatherStation - 1)
end

return self